# -*- coding: utf-8 -*-
import shutil

import xlrd
from google.protobuf.json_format import *

# -*- coding: utf-8 -*-
import csv
import os
import re
import time
from datetime import datetime, timedelta

from google.transit import gtfs_realtime_pb2

Mode = 'ver1'

# import gtfs_realtime_pb2

# カラーサンプル
color_sample = {'赤': 'FF0000', '青': '0000FF', '黄': 'FFFF00', '緑': '008000',
                '灰': 'A9A9A9', '黒': '000000', '橙': 'FFA500'}

# アラート
AlertCause = {'原因不明': gtfs_realtime_pb2.Alert.UNKNOWN_CAUSE,
              'その他': gtfs_realtime_pb2.Alert.OTHER_CAUSE,
              '船体の異常': gtfs_realtime_pb2.Alert.TECHNICAL_PROBLEM,
              'ストライキ': gtfs_realtime_pb2.Alert.STRIKE,
              'デモ活動': gtfs_realtime_pb2.Alert.DEMONSTRATION,
              '事故': gtfs_realtime_pb2.Alert.ACCIDENT,
              '祝日': gtfs_realtime_pb2.Alert.HOLIDAY,
              '天候': gtfs_realtime_pb2.Alert.WEATHER,
              '船舶のメンテナンス': gtfs_realtime_pb2.Alert.MAINTENANCE,
              '港付近の工事': gtfs_realtime_pb2.Alert.CONSTRUCTION,
              '警察による活動': gtfs_realtime_pb2.Alert.POLICE_ACTIVITY,
              '緊急の病人への対応': gtfs_realtime_pb2.Alert.MEDICAL_EMERGENCY
              }

AlertEffect = {'運休': gtfs_realtime_pb2.Alert.NO_SERVICE,
               '減便': gtfs_realtime_pb2.Alert.REDUCED_SERVICE,
               '深刻な遅れ': gtfs_realtime_pb2.Alert.SIGNIFICANT_DELAYS,
               '迂回': gtfs_realtime_pb2.Alert.DETOUR,
               '増便': gtfs_realtime_pb2.Alert.ADDITIONAL_SERVICE,
               'ダイヤの変更': gtfs_realtime_pb2.Alert.MODIFIED_SERVICE,
               'その他の影響': gtfs_realtime_pb2.Alert.OTHER_EFFECT,
               '影響不明': gtfs_realtime_pb2.Alert.UNKNOWN_EFFECT,
               '停泊港の変更': gtfs_realtime_pb2.Alert.STOP_MOVED
               }

########################################
# 航路データ読み取り用構造体
########################################
# 事業者情報
class cls_CorpInfo:
    def __init__(self):
        self.name = ''
        self.aid = ''
        self.president = ''
        self.zipcode = ''
        self.phone = ''
        self.address = ''
        self.fax = ''
        self.topurl = ''
        self.feeurl = ''
        self.email = ''


# 航路情報
class cls_KouroInfo:
    def __init__(self):
        self.name = ''
        self.url = ''
        self.color = ''
        self.book = ''
        self.route_id = ''  # これをRoute_idのキーとする（同一航路名対策）


# のりば情報
class cls_NoribaInfo:
    def __init__(self):
        self.name = ''
        self.pole = ''
        self.desc = ''
        self.lat = ''
        self.lon = ''
        self.url = ''
        self.yomi = ''
        self.en = ''
        self.ko = ''
        self.zh_cn = ''
        self.zh_tw = ''


# 港情報
class cls_Ports:
    def __init__(self):
        self.name = ''
        self.arv = ''
        self.dep = ''

    # ソート用
    def __lt__(self, other):
        # self < other
        return self.arv < other.arv

    def __repr__(self):
        return repr((self.name, self.arv, self.dep))


# 船舶情報
class cls_ShipInfo:
    def __init__(self):
        self.name = ''
        self.weight = ''
        self.power = ''
        self.engine_cnt = ''
        self.speed = ''
        self.person_cap = ''
        self.truck_cap = ''
        self.car_cap = ''
        self.launch = ''


# 運転日情報
class cls_UntenInfo:
    def __init__(self):
        self.name = ''
        self.kouro = ''  # 定義名が被るので路線ごとに分ける必要がある
        self.week = list()
        self.startdate = 0
        self.enddate = 0
        self.restdays = list()
        self.drivedays = list()  # 2020年.ver2で追加


# ダイヤ情報
class cls_DaiyaInfo:
    def __init__(self):
        self.name = ''
        self.day = ''
        self.ports = list()
        self.ship = ''  # 船体名：便名
        self.desc = ''
        self.kouro = ''
        self.ship_only = ''
        self.shape = ''


# 運賃情報
class cls_Untin:
    def __init__(self):
        self.port1 = ''
        self.port2 = ''
        self.kouro = ''
        self.untin = 0
        self.cabin_name = ''


# 車両・特殊手荷物情報
class cls_Tokusyu:
    def __init__(self):
        # 項目が多い為、番号でまとめた
        self.larges = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        self.comment1 = ''
        self.comment2 = ''
        self.kouro = ''
        self.ships = list()
        # ver2. ドライバー運賃を含むか否かと区分名をそのままIDにする
        self.includings = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        self.ids = list()


class cls_Alert:
    def __init__(self):
        self.route = ''  # 対象の路線名。EntitySelector/route_idを充てるのに使う。
        self.header = ''  # 見出し。header_textが該当
        self.description = ''  # 詳細説明。description_textが該当
        self.cause = 0  # 原因。cause。enumなので数値
        self.effect = 0  # 影響。effect。enumなので数値
        self.start = 0  # 開始日時。active_period。データ出力する時はPOSIX時間
        self.end = 0  # 終了日時。同上


class cls_stop_info:
    def __init__(self, col):
        self.code = col['stop_id']
        self.name = col['stop_name']
        self.zone = col['zone_id']


########################################
# 汎用関数
########################################
# 数値が時刻型になりうるかのチェック
def int_timecheck(hhmm):
    err = 0

    try:
        # 99時59分までは許容
        if hhmm > 9959:
            err = -1
            hhmm = -1
        else:
            sp = 2
            if len(str(hhmm)) == 3:
                sp = 1

            if len(str(hhmm)) < 3:
                hh = 0
                mm = hhmm
            else:
                hh = int(str(hhmm)[0:sp])
                mm = int(str(hhmm)[sp:])

            # 分60以上はNG
            if mm >= 60:
                err = -1
                hhmm = -1

        return err, hhmm
    except:
        return -1, hhmm


# Excelの時刻処理
# 文字列型（"900","09:00"）、数値型（900）、時刻型（0.xxx）に全てに対応
def excel_hhmm(cell):
    err = 0
    hhmm = -1

    try:
        if cell.value == '':
            return err, hhmm

        text = cell.value

        # 文字列の時
        if cell.ctype == 1:
            if ':' in text:  # 基本型 ex.9:00,09:00,09:00:00など
                hhmmss = text.split(':')
                # int型にキャスト出来ない場合などにexceptされる
                hhmm = int('{0:02}{1:02}'.format(int(hhmmss[0]), int(hhmmss[1])))
                # 時刻に適してるかチェック
                err, hhmm = int_timecheck(hhmm)
            elif '：' in text:  # 念の為、全角コロンも許容する
                hhmmss = text.split('：')
                # int型にキャスト出来ない場合などにexceptされる
                hhmm = int('{0:02}{1:02}'.format(int(hhmmss[0]), int(hhmmss[1])))
                # 時刻に適してるかチェック
                err, hhmm = int_timecheck(hhmm)
            else:  # ex.'900。不正書式だが、時刻になれば許可する
                # int型にキャスト出来ない場合はexceptされる
                hhmm = int(text)
                # 時刻に適してるかチェック
                err, hhmm = int_timecheck(hhmm)
        # 数字の時
        elif cell.ctype == 2:
            # 900など。不正書式だが、時刻になれば許容する
            err, hhmm = int_timecheck(int(text))
        # 時刻型の時
        elif cell.ctype == 3:
            dt = datetime(1899, 12, 30) + timedelta(days=text)
            # 24時間表記以上に対応する
            add_hour = int(timedelta(days=text).days * 24)
            hhmm = int('{0:02}{1:02}'.format(dt.hour + add_hour, dt.minute))

            err, hhmm = int_timecheck(hhmm)

        return err, hhmm
    except:
        return -1, -1


# GTFS形式の時刻(hh:mm:ssn)に変換
def gtfs_hhmmss(num):
    hhmm = '{0:04}'.format(num)
    hhmmss = '{0}:{1}:00'.format(hhmm[0:2], hhmm[2:4])
    return hhmmss


# 簡易的なエラーログため込み
def MakeError(mkflg, sheet_name, Errors):
    if mkflg == -1:
        Errors.append('シート「{0}」で不正な書式があります'.format(sheet_name))


# 重複港名に※マークをつける
def MakeCome(PortList):
    for i in range(len(PortList) - 1):
        port1 = PortList[i]

        # 既に※付きなら処理しない
        if '※' in port1:
            continue

        cnt = 0
        for j in range(i + 1, len(PortList)):
            port2 = PortList[j]

            if port1 == port2:
                cnt += 1
                PortList[j] = '{0}※{1}'.format(port1, cnt)

    return PortList


# 色コードが適切かどうかの判定
def CheckColor(color):
    try:
        rgb = [int(color[0:2], 16), int(color[2:4], 16), int(color[4:6], 16)]
        return True
    except:
        return False


# 色調反転
def MirrorColor(color):
    rgb = [int(color[0:2], 16), int(color[2:4], 16), int(color[4:6], 16)]

    for i in range(len(rgb)):
        rgb[i] = abs(rgb[i] - 255)

    html_color = '%02X%02X%02X' % (rgb[0], rgb[1], rgb[2])

    return html_color


# 指定の路線色コードの濃淡によりテキストカラー（白or黒）を返す
def setTextColor(color):
    text_color = '000000'
    rgb = [int(color[0:2], 16), int(color[2:4], 16), int(color[4:6], 16)]

    if ((rgb[0] * 299) + (rgb[1] * 587) + (rgb[2] * 114)) / 1000 < 127.5:
        text_color = 'FFFFFF'

    return text_color


# テキスト出力データの補正
def TextHosei(comment):
    # 改行コードを禁止
    r_com = comment.replace('\n', ' ')
    r_com = r_com.replace('\r', ' ')  # 不要なはずだが念のため

    # 半角カンマ防止
    if ',' in r_com:
        r_com = '"{0}"'.format(r_com)

    return r_com


# 日付け型変換判定
def CheckDate(yyyyMMdd):
    try:
        ymd = int(yyyyMMdd)
        dt = datetime.strptime(str(ymd), '%Y%m%d')
        return True
    except:
        return False


# trip_headsign作成
def MakeTripHeadsign(trip_times, n2id, id2n):
    headsign = list()
    prename = ''

    for i in range(1, len(trip_times)):
        pid = n2id[trip_times[i].name.split('※')[0]]
        port = id2n[pid]

        # 例えば港名が同じでのりば名が違うものが連続停泊するケースがある
        # 例）A~B～B～CをA~B~Cとする
        if prename != port:
            headsign.append(port)

        prename = port

    return TextHosei('～'.join(headsign))


# 列の名称取得
def GetColName(num):  # ZZまでは対応
    col = ''
    abc = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
           'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']

    try:
        if num == 0:
            return 'A'

        test = int(num / len(abc))

        if test > 0:
            col += abc[test - 1]

        test = int(num % len(abc))

        col += abc[test]

        return col
    except:
        return 'OVER Column'


# Excelのセルをstring型で取得する。文字列の場所に数値が入ってしまった場合の対策
def strCell(cell):
    # 文字列型の場合はそのままでOK
    if cell.ctype == 1:
        return cell.value
    # 数値型の時は想定外な数値になっているのを想定。小数点まであるので消す。
    if cell.ctype == 2:
        return str(int(cell.value))
    # 時刻型の時。数字/数字や数字:数字の時に発生しうる。
    if cell.ctype == 3:
        return str(cell.value)

    return cell.value


########################################
# シート読み込み関数
########################################
# 航路シート読み込み
def ReadKouro(sheet, book):
    cks = []
    err = 0
    ck = None
    route_title = ''

    # 項目名の下にデータ(i+1)がある想定
    try:

        for i in range(0, sheet.nrows):
            route_name = strCell(sheet.cell(i, 0))
            route_name_pattern = '^航路([0-9]*)$'
            result = re.match(route_name_pattern, route_name)
            if result:
                route_title = result.group(0)
                ck = cls_KouroInfo()
                ck.book = book
                if result.end() > 0:
                    # 01, 02, 03... の航路IDが入る
                    ck.route_id += result.group(1)
                else:
                    # ver3後方互換
                    ck.route_id = ''
                continue

            for j in range(0, sheet.ncols):
                text = sheet.cell(i, j).value

                if text == '航路URL':
                    ck.url = strCell(sheet.cell(i + 1, j))

                if text == '航路名':
                    ck.name = strCell(sheet.cell(i + 1, j))
                    if ck.name == '' and ck.url != '':
                        print('シート「{0}」{1}の航路名が空白です'.format(sheet.name, route_title))
                        err = -1
                    elif ck.name != '' and ck.url == '':
                        print('シート「{0}」{1}の航路URLが空白です'.format(sheet.name, route_title))
                        err = -1

                if text == '航路色':
                    color = strCell(sheet.cell(i + 1, j))

                    # 登録色を取得
                    if color in color_sample.keys():
                        ck.color = color_sample[color]
                    else:
                        ck.color = 'FFFFFF'

                        if color != '':  # 空白はデフォルトの白
                            # カラーコード判定
                            if CheckColor(color):
                                ck.color = color
                            else:
                                print('シート「{0}」{1}の航路色:{2} は不正です。基本色が使われます'.format(sheet.name, route_title, color))
                    if ck.name != '' and ck.url != '':
                        cks.append(ck)
                    ck = None
        return err, cks
    except IndexError:  # MEMO: 最終行が記入がないと存在しないのを考慮
        return err, cks
    except Exception as e:
        # print('Exception occurred while c   ode execution: ' + str(e))
        err = -1
        cks = []
        return err, cks


# のりばシート読み取り
def ReadNoriba(sheet):
    err = 0

    try:
        titles = sheet.row_values(1)
        noribas = list()

        for i in range(2, sheet.nrows):
            pi = cls_NoribaInfo()

            for j in range(0, sheet.ncols):
                text = strCell(sheet.cell(i, j))

                if titles[j] == '港名':
                    pi.name = text
                if titles[j] == '港よみ':
                    pi.yomi = text
                if titles[j] == '乗り場':
                    pi.pole = text
                if titles[j] == '港緯度':
                    pi.lat = sheet.cell(i, j).value
                if titles[j] == '港経度':
                    pi.lon = sheet.cell(i, j).value
                if titles[j] == '港説明':
                    pi.desc = TextHosei(text)
                if titles[j] == '港URL':
                    pi.url = text
                if titles[j] == '英語名':
                    pi.en = text
                if titles[j] == '繁体字名':
                    pi.zh_tw = text
                if titles[j] == '簡体字名':
                    pi.zh_cn = text
                if titles[j] == '韓国語名':
                    pi.ko = text

            # 名称が入っていない時は空白行とみなす
            if pi.name == '':
                print('シート「{0}」{1}行目 港名が入っていません。あるいは空白の行を作らないでください'.format(sheet.name, i + 1))
                err = -1

            # 読み、緯度、経度は必須
            if pi.yomi == '' or pi.lat == '' or pi.lon == '':
                print('シート「{0}」{1}行目 読み・緯度・経度は必須です'.format(sheet.name, i + 1))
                err = -1

            noribas.append(pi)

        # のりばが2つ未満ということはあり得ない
        if len(noribas) < 2:
            print('シート「{0}」 港名の定義が2つ未満です'.format(sheet.name))
            err = -1

        # 同じブックに2つの同名の港がある場合は運賃の関係上IDを作る必要がある
        ck_dic = dict()
        for ck in noribas:
            np = '{0}|{1}'.format(ck.name, ck.pole)

            if np not in ck_dic.keys():
                ck_dic[np] = 0
            else:
                ck_dic[np] += 1
                ck.pole = '{0}※{1}'.format(ck.pole, ck_dic[np])

        return err, noribas
    except:
        err = -1
        noribas = list()
        return err, noribas


# 船の紹介シート読み取り
def ReadShip(sheet, l_ships):
    err = 0

    try:
        for i in range(2, sheet.nrows):
            if sheet.cell(i, 0).value == '':
                continue

            cs = cls_ShipInfo()

            for j in range(0, sheet.ncols):
                try:
                    text = sheet.cell(i, j).value
                except IndexError:
                    break

                title = strCell(sheet.cell(1, j))

                if title == '船体名':
                    # 複数ブックで同じ船があるかもしれないので重複チェック
                    chk = next((x for x in l_ships if x.name == text), None)

                    if chk != None:
                        break

                    cs.name = text

                if title == '総トン数':
                    cs.weight = text

                if title == '主機関馬力':
                    if text != '':
                        cs.power = int(text)

                if title == '機関数':
                    if text != '':
                        cs.engine_cnt = int(text)

                if title == '航海速力':
                    if text != '':
                        cs.speed = int(text)

                if title == '旅客定員':
                    if text != '':
                        cs.person_cap = int(text)

                if title == 'トラック搭載可能車両数':
                    if text != '':
                        cs.truck_cap = int(text)

                if title == '乗用車搭載可能車両数':
                    if text != '':
                        cs.car_cap = int(text)

                if title == '就航年月':
                    cs.launch = text

                if title == '設備':
                    cs.equipment = text

            l_ships.append(cs)

        return err, l_ships

    except:
        err = -1
        return err, l_ships


# 運航日シート読み取り
def ReadUnten(sheet):
    listUnten = list()
    err = 0

    try:
        for i in range(3, sheet.nrows):
            cu = cls_UntenInfo()

            if sheet.cell(i, 0).value == '':
                continue

            for j in range(0, sheet.ncols):
                # 結合したセルが含まれたシートだと列カウントが正しく行えない為、エラーが出た時点で行の読込を終了する
                try:
                    text = strCell(sheet.cell(i, j))
                except IndexError:
                    break

                # 項目名はセル結合で充てに出来ない為、カラム番号決め打ちとする
                if j == 0:  # 定義名
                    cu.name = text
                    if cu.name == '':
                        print('シート「{0}」{1}行目 定義名が空白です'.format(sheet.name, i + 1))
                        err = -1
                if cu.name != '':  # 定義名が空の時はもう判定しない
                    if j >= 1 and j <= 8:  # 運航曜日群
                        if text == '○':
                            cu.week.append('1')
                        else:
                            cu.week.append('0')
                    if j == 9:  # 開始日
                        if CheckDate(text):
                            cu.startdate = int(text)
                        else:
                            print('シート「{0}」{1}行目 開始日:{2}は不正です'.format(sheet.name, i + 1, text))
                            err = -1
                    if j == 10:  # 終了日
                        if CheckDate(text):
                            cu.enddate = int(text)

                            # 開始＜終了以外は許容しない
                            if cu.startdate > cu.enddate:
                                print('シート「{0}」{1}行目 開始日と終了日の指定が誤っています\n 開始日:{2} 終了日:{3}'.format(
                                    sheet.name, i + 1, cu.startdate, cu.enddate))
                                err = -1
                        else:
                            print('シート「{0}」{1}行目 終了日:{2}は不正です'.format(sheet.name, i + 1, text))
                            err = -1
                    if j > 10:  # 以降運休日
                        if text != '':
                            if CheckDate(text):
                                cu.restdays.append(int(text))
                            else:
                                print('シート「{0}」{1}行目{2}列 運休日:{3}は不正です'.format(sheet.name, i + 1, GetColName(j), text))
                                err = -1

            listUnten.append(cu)

        # 運航日定義なし
        if len(listUnten) == 0:
            print('シート「{0}」 運航日の定義が1つもありません'.format(sheet.name))
            err = -1

        return err, listUnten
    except:
        err = -1
        listUnten = list()
        return err, listUnten


# ダイヤシート読み取り
def ReadDaiya(sheet, kouro_list, untenlist, ships):
    err = 0
    start_row = 0
    listUsed_list = list()
    listTrip_list = list()
    try:
        route_index = 0
        for kouro in kouro_list:
            route_index += 1
            listTrip = list()
            listUsed = list()
            PortList = list()

            # 順路取得
            diagram_name = '航路' + format(route_index, '02') + 'のダイヤ'
            if sheet.cell(start_row, 0).value != diagram_name:
                continue

            for j in range(1, sheet.ncols):
                if sheet.cell(2, j).value == '着':
                    port = sheet.cell(start_row + 1, j).value

                    if port != '':
                        PortList.append(port)

            ComePortList = MakeCome(PortList)

            for sheet_row in range(start_row + 3, sheet.nrows):
                if sheet.cell(sheet_row, 0).value == '':
                    # NOTE: 便名が無いので次の航路のダイヤへすすむ
                    start_row = sheet_row + 1
                    break

                cd = cls_DaiyaInfo()
                cd.kouro = kouro.name   # TODO: 航路名で良いのか？

                min_dep = 999999
                max_arv = 0
                for j in range(1, sheet.ncols):
                    port_index = int((j - 1) / 2)
                    if sheet.cell(start_row + 2, j).value == '着' and len(ComePortList) > port_index:
                        port = ComePortList[port_index]
                        s_err, arv = excel_hhmm(sheet.cell(sheet_row, j))
                        if s_err == -1:
                            print('シート「{0}」{1}行目{2}列 時刻:{3}は不正です'.format(sheet.name, sheet_row + 1, GetColName(j),
                                                                                     sheet.cell(sheet_row, j).value))
                            err = -1

                        s_err, dep = excel_hhmm(sheet.cell(sheet_row, j + 1))
                        if s_err == -1:
                            print('シート「{0}」{1}行目{2}列 時刻:{3}は不正です'.format(sheet.name, sheet_row + 1, GetColName(j + 1),
                                                                         sheet.cell(sheet_row, j + 1).value))
                            err = -1

                        # 時刻無し＝通過時は次へ
                        if dep == -1 and arv == -1:
                            continue

                        if dep != -1:
                            if min_dep > dep:
                                min_dep = dep

                        if arv != -1:
                            if max_arv < arv:
                                max_arv = arv

                        # 港名あり且ついずれかの時刻あり
                        if port != '':
                            port = ComePortList[int(j / 2)]

                            cp = cls_Ports()
                            cp.name = port

                            # 発か着いずれかのみの場合は同一時刻をいれておく
                            if dep == -1 and arv >= 0:
                                cp.dep = arv
                            else:
                                cp.dep = dep

                            if arv == -1 and dep >= 0:
                                cp.arv = dep
                            else:
                                cp.arv = arv

                            # 発着時刻の大小チェック。必ず発時刻>=着時刻となっているはず。
                            if cp.arv > cp.dep:
                                print('シート「{0}」{1}行目 港名:{2}\n 着時刻:{3} 発時刻:{4} 発着時刻が逆転しています'.format(
                                    sheet.name, sheet_row + 1, port.split('※')[0], sheet.cell(sheet_row, j).value,
                                    sheet.cell(sheet_row, j + 1).value))
                                err = -1

                            # 港/時刻を保存
                            cd.ports.append(cp)

                    # trip_short_nameとして扱う
                    if sheet.cell(start_row + 1, j).value == '使用船体':
                        if sheet.cell(sheet_row, j).value == '':
                            print('シート「{0}」{1}行目 使用船体が指定されていません'.format(sheet.name, sheet_row + 1))
                            err = -1
                            continue
                        else:
                            ship_chk = 0
                            for ship in ships:
                                if ship.name == sheet.cell(sheet_row, j).value:
                                    ship_chk = 1
                                    break

                            if ship_chk == 0:
                                print('シート「{0}」{1}行目 船体名 {2} は定義されていません'.format(sheet.name, sheet_row + 1,
                                                                                sheet.cell(sheet_row, j).value))
                                err = -1

                        if sheet.cell(sheet_row, 0).value == '':  # 便番号が空だが許容する
                            cd.ship = sheet.cell(sheet_row, j).value
                        else:
                            cd.ship = '{0}：{1}'.format(sheet.cell(sheet_row, j).value, sheet.cell(sheet_row, 0).value)

                        cd.ship_only = sheet.cell(sheet_row, j).value

                    if sheet.cell(start_row + 1, j).value == '運航日':
                        if len(cd.ports) != 0:  # 空行対策
                            cd.day = sheet.cell(sheet_row, j).value

                            # リスト入力し忘れの対策
                            if cd.day == '':
                                print('シート「{0}」{1}行目 運航日が指定されていません'.format(sheet.name, sheet_row + 1))
                                err = -1
                                continue

                            # リスト入力規則なのでないはずだが念のため
                            daycheck = next((x for x in untenlist if x.name == cd.day), None)

                            if daycheck == None:
                                print('シート「{0}」{1}行目 指定された運航日"{2}"は定義されていません'.format(sheet.name, sheet_row + 1, cd.day))
                                err = -1
                                continue

                            # 運転日シートに記載があってもダイヤ側で使われてない可能性がある為、
                            # 実際に使用した運転日のみを記録しておく
                            uu = ''
                            if Mode == 'ver1':
                                uu = '{0}'.format(cd.day)
                            if Mode == 'ver2':
                                uu = cd.day

                            if uu not in listUsed:
                                listUsed.append(uu)
                    # 注記
                    if sheet.cell(start_row + 1, j).value == '注記':
                        cd.desc = TextHosei(sheet.cell(sheet_row, j).value)

                    # 拡張shapes指定
                    if sheet.cell(start_row + 1, j).value == '航海ルート':
                        cd.shape = TextHosei(sheet.cell(sheet_row, j).value)

                if min_dep > max_arv:
                    err = -1
                    print('シート「{0}」{1}行目 入力された発時刻と着時刻が矛盾しています'.format(sheet.name, sheet_row + 1))
                    continue

                # 便保存
                listTrip.append(cd)

            # 便情報なし
            if len(listTrip) == 0:
                print('シート「{0}」{1}に有効な便が1つもありません'.format(sheet.name, kouro.name))
                err = -1

            listTrip_list.append(listTrip)
            listUsed_list.append(listUsed)

            for sheet_row in range(start_row, sheet.nrows):
                if sheet.cell(sheet_row, 0).value != '':
                    start_row = sheet_row
                    break

        # 使用済み運転フラグと便を返却
        return err, listUsed_list, listTrip_list
    except Exception as e:
        print('Exception occurred while code execution: ' + str(e))
        err = -1
        listUsed_list = list()
        listTrip_list = list()

        return err, listUsed_list, listTrip_list


# 運賃シート読み取り
def ReadUntin(sheet, kouro_list, dia_list):
    err = 0
    listFare_list = list()
    try:
        before_row = 0
        for i in range(0, len(kouro_list)):
            PortList = list()
            listFare = list()
            ports = dia_list[i][0].ports
            for port in ports:
                PortList.append(port.name)

            # ※付きのリストを生成する（共通）
            ComeList = MakeCome(PortList)
            kouro = kouro_list[i]
            # 一つ目の運賃
            listNum = 1
            for j in range(0, sheet.ncols - 1):
                port1 = ''
                col = j
                # 客室名
                cabin_name = sheet.cell((sheet.ncols + 3) * listNum - 2 + before_row, 1).value
                for i in range(col + 1, sheet.ncols):
                    row = i + ((sheet.ncols + 3) * (listNum - 1)) + before_row

                    untin = sheet.cell(row, col).value

                    if untin == '':
                        break  # 「 - 」の部分読み飛ばし

                    elif col + 1 == i:  # 港名にあたった時@タイトル行の影響でずれるのを補正
                        port1 = sheet.cell(row, col).value

                        if port1 != '':
                            port1 = ComeList[col]

                    elif untin != '':
                        port2 = sheet.cell(i, i - 1).value
                        untin = sheet.cell(row, j).value

                        if port2 != '':  # 終了
                            if untin != '':
                                port2 = ComeList[i - 1]  # kome
                                cu = cls_Untin()
                                cu.port1 = port1.split('※')[0]
                                cu.port2 = port2.split('※')[0]
                                cu.untin = int(untin)
                                cu.kouro = kouro.name
                                cu.cabin_name = cabin_name

                                listFare.append(cu)
                        else:
                            break
            # 二つ目の運賃
            listNum = 2
            last_row = 0
            for j in range(0, sheet.ncols - 1):
                port1 = ''
                col = j
                # 客室名
                last_row = (sheet.ncols + 3) * listNum - 2 + before_row
                cabin_name = sheet.cell(last_row, 1).value
                last_row += 1
                for i in range(col + 1, sheet.ncols):
                    row = i + ((sheet.ncols + 3) * (listNum - 1)) + before_row

                    untin = sheet.cell(row, col).value

                    if untin == '':
                        break  # 「 - 」の部分読み飛ばし

                    elif col + 1 == i:  # 港名にあたった時@タイトル行の影響でずれるのを補正
                        port1 = sheet.cell(row, col).value

                        if port1 != '':
                            port1 = ComeList[col]

                    elif untin != '':
                        port2 = sheet.cell(i, i - 1).value
                        untin = sheet.cell(row, j).value

                        if port2 != '':  # 終了
                            if untin != '':
                                port2 = ComeList[i - 1]  # kome
                                cu = cls_Untin()
                                cu.port1 = port1.split('※')[0]
                                cu.port2 = port2.split('※')[0]
                                cu.untin = int(untin)
                                cu.kouro = kouro.name
                                cu.cabin_name = cabin_name

                                listFare.append(cu)
                        else:
                            break
            listFare_list.append(listFare)
            for k in range(last_row, sheet.nrows):
                if sheet.cell(k, 0).value != '':
                    before_row = k
                    break
    except IndexError:
        return err, listFare_list
    except Exception as e:
        print('Exception occurred while code execution: ' + str(e))
        err = -1
        listFare_list = list()

    return err, listFare_list

# 車両・特殊手荷物シート読み取り
def ReadTokusyu(sheet, kouro_list, dialist_list):
    err = 0
    ct_list = list()
    start_row = 0
    try:
        for i in range(0, len(kouro_list)):
            ships = set()
            inc_col = -1
            dialist = dialist_list[i]
            # 区分名は固定とするので行番号固定としてデータを処理する。
            ct = cls_Tokusyu()
            kouro = kouro_list[i].name
            ct.kouro = kouro

            for dia in dialist:
                ships.add(dia.ship_only)

            # ver2のシートかどうか判定しておく
            for i in range(sheet.ncols):
                cell = sheet.cell(1, i).value

                if cell == 'ドライバー船賃':
                    inc_col = i
                    break

            for i in range(start_row + 2, start_row + len(ct.larges) + 2):
                cell = sheet.cell(i, 1)

                # 項目名をIDとする。厳密にはユニークにならない
                ct.ids.append('{0}|{1}'.format(kouro, sheet.cell(i, 0).value))

                if cell.ctype == 1:  # 文字列の時
                    if cell.ctype != '':
                        ct.larges[i - 2] = -1

                if cell.ctype == 2:  # 数値の時
                    ct.larges[i - 2 - start_row] = int(cell.value)

                if inc_col >= 0:
                    if sheet.cell(i, inc_col).value == "含まない":
                        ct.includings[i - 2 - start_row] = 1

            ct.comment1 = sheet.cell(start_row + 2, 2).value
            ct.comment2 = sheet.cell(start_row + 14, 2).value

            ct.comment1 = TextHosei(ct.comment1)
            ct.comment2 = TextHosei(ct.comment2)

            # 船体情報を紐づける
            ct.ships = list(ships)

            ct_list.append(ct)

            for start_row in range(i + 1, sheet.nrows):
                if sheet.cell(start_row, 1).value != '':
                    start_row -= 1
                    break
    except IndexError:
        return err, ct_list
    except Exception as e:
        print('Exception occurred while code execution: ' + str(e))
        err = -1
        ct_list = list()

    return err, ct_list


# 運転日ver2読み取り。外部設定ファイル
def ReadUnten2(path):
    ListUnten = list()
    err = 0

    try:
        with open(path, 'rt') as rf:
            for row in csv.reader(rf):
                cu = cls_UntenInfo()

                cu.name = row[0]
                cu.kouro = 'all'  # 全Excelファイル共通とするため
                cu.week = ['0', '0', '0', '0', '0', '0', '0', '1']  # 運休扱いにならないよう祝日は1を立てる
                cu.startdate = int(row[1])  # 曜日分割しないなら不要
                cu.enddate = int(row[2])  # 同上

                driveData = row[3].split('，')

                for dd in driveData:
                    dx = dd.split('/')

                    for i in range(len(dx[2])):
                        if dx[2][i] == '1':
                            ymd = '{0}{1}{2:02d}'.format(dx[0], dx[1], i + 1)
                            cu.drivedays.append(ymd)

                ListUnten.append(cu)

        return err, ListUnten
    except:
        err = -1
        ListUnten = list()
        return err, ListUnten


def ReadAlert(sheet, b_KouroList):
    ListAlert = list()
    err = 0
    init_col = 0

    try:
        for i in range(sheet.nrows):
            if sheet.cell(i, 0).value == '見出し':
                continue
            if sheet.cell(i, 0).value == '航路名':
                init_col = 1
                continue

            for j in range(4 + init_col):
                if sheet.cell(i, j).value == '':
                    err = -1
                    print('{0}行目の{1}が入力されていません'.format(i, sheet.cell(0, j).value))

            if sheet.cell(i, 4 + init_col).value == '' and sheet.cell(i, 5 + init_col).value == '':
                err = -1
                print('{0}行目の開始日と終了日が入力されていません'.format(i))

            if sheet.cell(i, 4 + init_col).value != '' and sheet.cell(i, 5 + init_col).value != '':
                if int(sheet.cell(i, 4 + init_col).value) > int(sheet.cell(i, 5 + init_col).value):
                    err = -1
                    print('{0}行目の開始日と終了日が逆転しています'.format(i))

            if err < 0:
                continue

            ca = cls_Alert()
            if init_col > 0:
                ca.route = TextHosei(sheet.cell(i, 0).value)
            else:
                ca.route = b_KouroList[0].route_id
            ca.header = TextHosei(sheet.cell(i, 0 + init_col).value)
            ca.description = TextHosei(sheet.cell(i, 1 + init_col).value)

            ca.cause = AlertCause[TextHosei(sheet.cell(i, 2 + init_col).value)]
            ca.effect = AlertEffect[TextHosei(sheet.cell(i, 3 + init_col).value)]

            # 開始日終了日はどちらかがセットされていればOK
            st = ''
            ed = ''
            if sheet.cell(i, 4 + init_col).value != '':
                st = '{0}'.format(int(sheet.cell(i, 4 + init_col).value))

                if CheckDate(st):
                    # 開始日の0時にセット
                    st = '{0} 00:00:00'.format(st)
                    dt = datetime.strptime(st, '%Y%m%d %H:%M:%S')
                    ca.start = int(time.mktime(dt.timetuple()))
                else:
                    print('{0}行目の開始日の日付けが正しくありません'.format(i))
                    err = -1

            if sheet.cell(i, 5).value != '':
                ed = '{0}'.format(int(sheet.cell(i, 5 + init_col).value))

                if CheckDate(ed):
                    # 終了日の23時59分59秒にセット
                    ed = '{0} 23:59:59'.format(ed)
                    dt = datetime.strptime(ed, '%Y%m%d %H:%M:%S')
                    ca.end = int(time.mktime(dt.timetuple()))
                else:
                    print('{0}行目の終了日の日付けが正しくありません'.format(i))
                    err = -1

            if st == '' and ed == '':
                print('{0}行目、開始日と終了日はいずれかが必須です')
                err = -1
                continue

            if err < 0:
                continue

            ListAlert.append(ca)

        return err, ListAlert
    except Exception as e:
        print('Exception occurred while code execution: ' + str(e))
        err = -1
        ListAlert = list()
        return err, ListAlert

def ReadUntenTokutei(sheet, b_UntenList):
    err = 0

    try:
        for i in range(2, sheet.nrows):
            if sheet.cell(i, 0).value == '':
                continue
            for drive in b_UntenList:
                if drive.name == sheet.cell(i, 0).value:
                    if CheckDate(sheet.cell(i, 1).value):
                        st = '{0} 00:00:00'.format(int(sheet.cell(i, 1).value))
                        dt = datetime.strptime(st, '%Y%m%d %H:%M:%S')
                        drive.drivedays.append('{0}{1}{2:02d}'.format(dt.year, dt.month, dt.day))
                        break
                    else:
                        print('{0}行目の開始日の日付けが正しくありません'.format(i))
                        err = -1
                        break
        return err, b_UntenList
    except Exception as e:
        print('Exception occurred while code execution: ' + str(e))
        err = -1
        b_UntenList = list()
        return err, b_UntenList

########################################
# 追加補正処理
########################################

# 2巡目以降の港ＩＤの補正
def ComePortDelete(output):
    key2sids = dict()
    zid2key = dict()

    # stops読み込み。航路標準では親停留所は存在しないため、※つきか否かを名称と座標で判定する
    with open('{0}/stops.txt'.format(output), 'rt', encoding='utf-8') as rf:
        for col in csv.DictReader(rf):
            key = '{0}|{1}|{2}'.format(col['stop_name'], col['stop_lat'], col['stop_lon'])

            if key not in key2sids.keys():
                key2sids[key] = list()

            key2sids[key].append(cls_stop_info(col))
            zid2key[col['zone_id']] = key

    fid2fare = dict()

    # fare_attributesを参照
    with open('{0}/fare_attributes.txt'.format(output), 'rt', encoding='utf-8') as rf:
        for col in csv.DictReader(rf):
            fid2fare[col['fare_id']] = col['price']

    del_flg = True

    key2fare = dict()
    # fare_rulesを参照
    with open('{0}/fare_rules.txt'.format(output), 'rt', encoding='utf-8') as rf:
        for col in csv.DictReader(rf):
            key = '{0}|{1}|{2}'.format(col['route_id'], zid2key[col['origin_id']], zid2key[col['destination_id']])

            if key not in key2fare.keys():
                key2fare[key] = fid2fare[col['fare_id']]
            else:
                # ※付きとそうではない港同士の運賃で不一致が存在する場合は統合できない
                if key2fare[key] != fid2fare[col['fare_id']]:
                    del_flg = False
                    break

    # 統合処理（※つき削除が認められる場合）
    if del_flg:
        reps = dict()
        # フォーマットの仕様上、先に出てきているのが※なしを前提
        for key in key2sids.keys():
            if len(key2sids[key]) > 1:
                for i in range(1, len(key2sids[key])):
                    reps[key2sids[key][i].code] = key2sids[key][0].code

        ofiles = os.listdir(output)

        for of in ofiles:
            data = open('{0}/{1}'.format(output, of), 'rt', encoding='utf-8')
            save = open('{0}/_{1}'.format(output, of), 'wt', encoding='utf-8', newline='\n')

            reader = csv.DictReader(data)
            writer = csv.DictWriter(save, reader.fieldnames, lineterminator="\n")
            writer.writerow(dict((v, v) for v in reader.fieldnames))

            # 航路程度なので許容するが全データを参照するため効率は悪い
            depchk = set()

            for col in reader:
                dep = list()
                for item in reader.fieldnames:
                    for key in reps.keys():
                        if key in col[item]:
                            col[item] = col[item].replace(key, reps[key])

                    dep.append(col[item])

                depline = ','.join(dep)

                # stopsやfare_attributes,fare_rulesなど置換により重複するデータを削除する
                # stop_timesはtripsの関係上重複はしないので問題なし
                if depline not in depchk:
                    writer.writerow(col)
                    depchk.add(depline)

            data.close()
            save.close()

            os.remove('{0}/{1}'.format(output, of))
            os.rename('{0}/_{1}'.format(output, of), '{0}/{1}'.format(output, of))
    else:
        print('2回目以降の停泊時に異なる金額となる区間があるため、港IDの統合処理は実施しません')

        # import pdb; pdb.set_trace()


def mk_headsign(output):
    tid2order = dict()

    with open('{0}/stop_times.txt'.format(output), 'rt', encoding='utf-8') as rf:
        for col in csv.DictReader(rf):
            tid = col['trip_id']

            if col['trip_id'] not in tid2order.keys():
                tid2order[tid] = list()

            tid2order[tid].append(col['stop_id'])

    sid2name = dict()
    with open('{0}/stops.txt'.format(output), 'rt', encoding='utf-8') as rf:
        for col in csv.DictReader(rf):
            sid2name[col['stop_id']] = col['stop_name']

    tid2mid = dict()
    for tid in tid2order.keys():
        # ※つきＩＤは消しているが、運賃の都合上統合されないものがあるので名称合致とする
        if sid2name[tid2order[tid][0]] == sid2name[tid2order[tid][-1]]:
            # 中間地点を仮の行先として保存しておく
            mid = int(len(tid2order[tid]) / 2)
            mid_sid = tid2order[tid][mid]

            # 念のため、中間地点が最終行先と同じ場合は1つずらす
            if sid2name[mid_sid] == sid2name[tid2order[tid][-1]]:
                mid_sid = tid2order[tid][mid - 1]

            tid2mid[tid] = mid_sid

    data = open('{0}/stop_times.txt'.format(output), 'rt', encoding='utf-8')
    save = open('{0}/_stop_times.txt'.format(output), 'wt', encoding='utf-8', newline='\n')

    reader = csv.DictReader(data)
    writer = csv.DictWriter(save, reader.fieldnames, lineterminator="\n")
    writer.writerow(dict((v, v) for v in reader.fieldnames))

    for col in reader:
        if col['trip_id'] in tid2mid.keys():
            num = int(len(tid2order[col['trip_id']]) / 2) - 1

            if num >= int(col['stop_sequence']):
                col['stop_headsign'] = '{0}行'.format(sid2name[tid2mid[col['trip_id']]])

        writer.writerow(col)

    data.close()
    save.close()

    os.remove('{0}/stop_times.txt'.format(output))
    os.rename('{0}/_stop_times.txt'.format(output), '{0}/stop_times.txt'.format(output))


########################################
# メイン処理開始
########################################

corp_master = ''
target = ''
output = ''
ver2dir = ''
setfile = '標準フォーマットコンバータ設定ファイル.txt'

f_err = 0
f_t = -1
f_c = -1
f_o = -1
# 設定ファイルの有無で動作を変更する
if not os.path.exists(setfile):
    # ないときは引数設定
    if len(sys.argv) > 3:
        corp_master = sys.argv[1]
        target = sys.argv[2]
        output = sys.argv[3]
    else:  # 引数もなければ終了
        print('{0}が見つかりません。処理を終了します'.format(setfile))
        sys.exit()
else:
    sflg = -1
    data = open(setfile, 'rt', encoding="utf-8")

    for row in data:
        if len(row) == 0:
            continue
        if row.startswith(';'):
            continue

        if '=' in row:
            numfind = row.find('=')
            item = row[0:numfind]
            name = row[numfind + 1:].replace('\n', '')

            if item == '対象航路データフォルダ':
                target = name
                f_t = 0
            if item == '事業者情報':
                corp_master = name
                f_c = 0
            if item == '出力フォルダ':
                output = name
                f_o = 0
            if item == 'ver2対応外部ファイル':
                ver2dir = name
    data.close()

if f_t == -1:
    print(' >対象航路データフォルダが指定されていません')
    f_err = -1
if f_c == -1:
    print(' >事業者情報ファイルが指定されていません')
    f_err = -1
if f_o == -1:
    print(' >出力フォルダが指定されていません')
    f_err = -1

if f_err == -1:
    print('\n{0}で上記の設定をしてください。処理を終了します'.format(setfile))
    sys.exit()

if not os.path.exists(corp_master):
    print(' >事業者情報={0}見つかりません'.format(corp_master))
    f_err = -1

if not os.path.exists(target):
    print(' >対象航路データフォルダ={0}が見つかりません'.format(target))
    f_err = -1

if target == output:
    print(' >{0} 入力フォルダと出力フォルダの名称は同じにしないでください'.format(target))
    f_err = -1

if ver2dir != '':
    if not os.path.exists(ver2dir):
        print('追加外部ファイルのディレクトリ={0}が見つかりません'.format(ver2dir))
        f_err = -1

# syukujitsu.csv存在チェック
if not os.path.exists('./syukujitsu.csv'):
    print(' >syukujitsu.csvがありません')
    f_err = -1

if f_err == -1:
    print('\n{0}で上記の確認をしてください。処理を終了します'.format(setfile))
    sys.exit()

# 祝日判定モジュール読み取り
holidays  = list()

# shift-jis のCSVファイル
with open('./syukujitsu.csv', 'r', encoding='shift_jis') as csvfile:
    csv_reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    next(csv_reader)  # ヘッダ行をスキップ
    for row in csv_reader:
        d = row[0].split('/')
        holidays.append('{0}{1:02}{2:02}'.format(int(d[0]), int(d[1]), int(d[2])))

# 各種BOOK、GTFSデータのオープンチェック
OPEN_ERROR = list()
# Excel
CheckExcels = list()
CheckExcels.append(corp_master)
exs = os.listdir(target)

for ex in exs:
    path = '{0}/{1}'.format(target, ex)
    CheckExcels.append(path)

for xlsx in CheckExcels:
    if xlsx.endswith('xlsx') or xlsx.endswith('xls'):
        try:
            book = xlrd.open_workbook(xlsx)
        except:
            import traceback

            traceback.print_exc()
            # OPEN_ERROR.append('{0}'.format(xlsx))
            # continue

if os.path.exists(output):
    gtfs = os.listdir(output)

    for file in gtfs:
        try:
            data = open('{0}/{1}'.format(output, file), 'wt', encoding='utf-8')
            data.close()
        except:
            OPEN_ERROR.append('{0}/{1}'.format(output, file))
            continue

if len(OPEN_ERROR) > 0:
    print('以下の入出力ファイルが開かれています。閉じてから実行してください\n')

    for oe in OPEN_ERROR:
        print(' >{0}'.format(oe))

    sys.exit()

# 事業者情報ブック読み取り
book = xlrd.open_workbook(corp_master)
sheet_names = book.sheet_names()

mkflg = 0
if '事業者情報' not in sheet_names:
    print('シート「事業者情報」がありません。処理を終了します\n')
    sys.exit()

ci = ''
try:
    sheet = book.sheet_by_name('事業者情報')
    ci = cls_CorpInfo()

    for i in range(0, sheet.nrows):
        item = sheet.cell(i, 0).value
        text = sheet.cell(i, 1).value

        if item == '事業者名':
            ci.name = text
        if item == '法人番号':
            ci.aid = text

            # 法人番号が数字の場合、Excelの仕様上、小数点が入ってしまう
            if sheet.cell(i, 1).ctype == 2:
                ci.aid = int(text)

        if item == '代表者名':
            ci.president = text
        if item == '郵便番号':
            ci.zipcode = text
        if item == '本社住所':
            ci.address = text
        if item == '電話番号':
            ci.phone = text
        if item == 'FAX番号':
            ci.fax = text
        if item == '航路ホームページ':
            ci.topurl = text
        if item == '乗船券購入ページ':
            ci.feeurl = text
        if item == '事業者Eメールアドレス':
            ci.email = text
except:  # 何も入力されていない場合は終了
    pass

if ci.name == '' or ci.aid == '':
    print('{0}：事業者名と法人番号は必須項目です。処理を終了します\n'.format(corp_master))
    sys.exit()

# 対象航路データ
files = os.listdir(target)

if len(files) == 0:
    print('対象となるファイルが一つも存在しません。処理を終了します\n')
    sys.exit()

# GTFSとしては、船体不明、運賃なし、車両・特殊手荷物は無くても仕様上OKだが、
# フェリーフォーマットとしては特殊手荷物以外は必須。Excel上で保護しているのでないことはないが解除もできるため確認しておく。
mustSheets = ['①航路', '②のりば', '③船の紹介', '④運航日', '⑤ダイヤ', '⑥旅客運賃']
ReadSheets = ['①航路', '②のりば', '③船の紹介', '④運航日', '⑤ダイヤ', '⑥旅客運賃', '⑦車両・特殊手荷物', '⑧アラート', '⑨運航日_特定日']

# 全ブックレベルの結果格納。GTFSのマスター。各ブックでNGがない場合のみ収録する。
KouroList = list()
NoribaList = list()
UntenList = list()
DaiyaList = list()
UntinList = list()
TokusyuList = list()
UsedUnten = list()
AlertList = list()
ShipList = list()

AlertFlg = 0

UntenFileVer2 = '{0}/運転日定義.txt'.format(ver2dir)

# 2020年Ver2対応
UntenMaster = None
if os.path.exists(UntenFileVer2):
    Mode = 'ver2'
    err, UntenMaster = ReadUnten2(UntenFileVer2)

# 各ファイル読み取り
for file in files:
    f_name, ext = os.path.splitext(file)

    print('■ブック名：{0}を読み込み開始'.format(file))

    # Excelファイルではないものは無視する
    if ext != '.xlsx' and ext != '.xls':
        continue

    path = '{0}/{1}'.format(target, file)

    book = xlrd.open_workbook(path)
    sheet_names = book.sheet_names()

    mkflg = 0

    # 各ブックレベルの結果格納
    b_KouroList = list()
    b_NoribaList = list()
    b_UntenList = list()
    b_DaiyaList = list()
    b_UntinList = list()
    b_TokusyuList = list()
    b_UsedUnten = list()
    b_Alert = list()
    Error = list()

    # 必須ファイルの存在チェック
    for mst in mustSheets:
        if mst not in sheet_names:
            print('必須シート「{0}」がありません'.format(mst))
            mkflg = -1

    if mkflg == -1:
        print('\n標準フォーマットデータ化はスキップされます')
        continue

    # 各種シート読込。シートの順が変更されてても問題ないようにループ順は固定
    kouro_name = ''
    for sn in ReadSheets:
        # 必須でないものはスキップ
        if sn not in sheet_names:
            continue

        sheet = book.sheet_by_name(sn)

        if sn == '①航路':
            mkflg, b_KouroList = ReadKouro(sheet, f_name)
            MakeError(mkflg, sn, Error)
        if sn == '②のりば':
            mkflg, b_NoribaList = ReadNoriba(sheet)
            MakeError(mkflg, sn, Error)
        if sn == '③船の紹介':
            mkflg, ShipList = ReadShip(sheet, ShipList)
            MakeError(mkflg, sn, Error)
        if sn == '④運航日':
            if Mode == 'ver1':
                mkflg, b_UntenList = ReadUnten(sheet)
                MakeError(mkflg, sn, Error)
            if Mode == 'ver2':
                b_UntenList = UntenMaster
        if sn == '⑤ダイヤ':
            mkflg, b_UsedUnten, b_DaiyaList = ReadDaiya(sheet, b_KouroList, b_UntenList, ShipList)
            MakeError(mkflg, sn, Error)
        if sn == '⑥旅客運賃':
            mkflg, b_UntinList = ReadUntin(sheet, b_KouroList, b_DaiyaList)
            MakeError(mkflg, sn, Error)
        if sn == '⑦車両・特殊手荷物':
            mkflg, b_TokusyuList = ReadTokusyu(sheet, b_KouroList, b_DaiyaList)
            MakeError(mkflg, sn, Error)
        if sn == '⑧アラート':
            mkflg, b_Alert = ReadAlert(sheet, b_KouroList)
            MakeError(mkflg, sn, Error)
        if sn == '⑨運航日_特定日':
            mkflg, b_UntenList = ReadUntenTokutei(sheet, b_UntenList)
            MakeError(mkflg, sn, Error)

    # 全シート問題なかったらGTFSマスターに登録
    if len(Error) == 0:
        KouroList.extend(b_KouroList)
        NoribaList.extend(b_NoribaList)
        UntenList.extend(b_UntenList)
        DaiyaList.extend(b_DaiyaList)
        UntinList.extend(b_UntinList)
        TokusyuList.extend(b_TokusyuList)
        UsedUnten.extend(b_UsedUnten)
        AlertList.extend(b_Alert)

        print(' >読み込み完了')
    else:
        print('\n不正な書式のシートが確認されました。\n標準フォーマットデータ化はスキップされます')

        for er in Error:
            print(' >{0}'.format(er))
            AlertFlg = 1

    print('\n')

    ########################################
    # GTFS作成処理開始
    ########################################
    print('■標準フォーマットデータ作成開始')
    # 出力ディレクトリ
    if os.path.exists(output):
        shutil.rmtree(output)

        # ディレクトリが消える前にディレクトリを作成されることがあるので、待機をいれておく
        time.sleep(1)

    os.mkdir(output)

    # agency.txt
    data = open('{0}/agency.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'agency_id,agency_name,agency_url,agency_timezone,agency_lang,agency_phone,agency_fare_url,agency_email\n')
    data.write('{0},{1},{2},{3},{4},{5},{6},{7}\n'.format(
        ci.aid, TextHosei(ci.name), ci.topurl, 'Asia/Tokyo', 'ja', ci.phone, ci.feeurl, ci.email))
    data.close()

    # stops.txt & translations.txt
    data = open('{0}/stops.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'stop_id,stop_code,stop_name,stop_desc,stop_lat,stop_lon,zone_id,stop_url,location_type,parent_station,stop_timezone,wheelchair_boarding,level_id,platform_code\n')

    trans = open('{0}/translations.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    trans.write('trans_id,lang,translation\n')

    port2id = dict()
    savedsid = list()
    usedname = list()
    id2port = dict()  # trip_headsign用(港名+のりば→港名のみに変換する為、港名+のりば→ID→港名を保持)

    for i in range(len(NoribaList)):
        si = NoribaList[i]
        portname = '{0}{1}'.format(si.name, si.pole)

        if portname in savedsid:
            continue

        savedsid.append(portname)

        # portID = 'S{0:03}'.format(i+1)
        portID = '[{0:02}]{1}'.format(i + 1, portname.split('※')[0])

        port2id[portname] = portID
        id2port[portID] = si.name

        data.write('{0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11},,\n'.format(
            TextHosei(portID), '', TextHosei(si.name), TextHosei(si.pole.split('※')[0]),
            si.lat, si.lon, TextHosei(portID), TextHosei(si.url), '0', '', 'Asia/Tokyo', '0'))

        # ※付き以外は翻訳出力
        if si.name not in usedname:
            # 名称と読みは必須
            trans.write('{0},{1},{2}\n'.format(TextHosei(si.name), 'ja', TextHosei(si.name)))
            trans.write('{0},{1},{2}\n'.format(TextHosei(si.name), 'ja-HrKt', TextHosei(si.yomi)))

            # 翻訳は任意
            if si.en != '':
                trans.write('{0},{1},{2}\n'.format(TextHosei(si.name), 'en', TextHosei(si.en)))

            if si.ko != '':
                trans.write('{0},{1},{2}\n'.format(TextHosei(si.name), 'ko', TextHosei(si.ko)))

            if si.zh_cn != '':
                trans.write('{0},{1},{2}\n'.format(TextHosei(si.name), 'zh-Hans', TextHosei(si.zh_cn)))

            if si.zh_tw:
                trans.write('{0},{1},{2}\n'.format(TextHosei(si.name), 'zh-Hant', TextHosei(si.zh_tw)))

            usedname.append(si.name)
    data.close()
    trans.close()

    # routes.txt
    data = open('{0}/routes.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'route_id,agency_id,route_short_name,route_long_name,route_desc,route_type,route_url,route_color,route_text_color\n')

    kouro2id = dict()
    for i in range(len(KouroList)):
        ri = KouroList[i]
        # routeID = 'R{0:03}'.format(i+1)
        routeID = '[{0:02}]{1}'.format(i + 1, ri.name)

        kouro2id[ri.name] = routeID
        data.write('{0},{1},{2},{3},{4},{5},{6},{7},{8}\n'.format(
            TextHosei(routeID), ci.aid, '', TextHosei(ri.name), '', '4', TextHosei(ri.url), ri.color,
            setTextColor(ri.color)))
    data.close()

    # calendar.txt & calendar_dates.txt
    if Mode == 'ver2':
        UntenList = UntenMaster

    unten2id = dict()
    MaxDate = 0
    MinDate = 30000101
    data1 = open('{0}/calendar.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data2 = open('{0}/calendar_dates.txt'.format(output), 'wt', encoding='utf-8', newline='\n')

    data1.write('service_id,monday,tuesday,wednesday,thursday,friday,saturday,sunday,start_date,end_date\n')
    data2.write('service_id,date,exception_type\n')

    cnt = 1
    chkDup = list()
    for i in range(len(UntenList)):
        ui = UntenList[i]
        uiCode = ''
        if Mode == 'ver1':
            uiCode = '{1}'.format(ui.kouro, ui.name)
        if Mode == 'ver2':
            uiCode = ui.name

        # ダイヤ上使用していない運転日データは出力しないようにする
        unused = True
        for used in UsedUnten:
            if uiCode in used:
                unused = False
        if unused:
            continue

        stdt = datetime.strptime(str(ui.startdate), '%Y%m%d')
        eddt = datetime.strptime(str(ui.enddate), '%Y%m%d')
        days = (eddt - stdt).days

        # FeedInfo用の有効期間
        if MaxDate < ui.enddate:
            MaxDate = ui.enddate

        if MinDate > ui.startdate:
            MinDate = ui.startdate

        # 運休日が開始～終了の期間外の時は無視するために新しいリストに付け替える
        nRestdays = list()
        for rest in ui.restdays:
            if int(rest) < ui.startdate or int(rest) > ui.enddate:
                pass
            else:
                nRestdays.append(rest)

        # 祝日が運転になっていない時は祝日計算をして運休日に足す
        if ui.week[7] == '0':
            for j in range(days):
                chkdt = stdt + timedelta(days=j)

                ymd = '{0}{1:02}{2:02}'.format(int(chkdt.year), int(chkdt.month), int(chkdt.day))
                if ymd in holidays:
                    ymd = int(ymd)
                    if ymd not in nRestdays:
                        nRestdays.append(ymd)

        # コードより文字列の方がわかりやすいので「航路名|運転日名」とする
        # sid = 'C{0:04}'.format(cnt)
        sid = uiCode
        unten2id[uiCode] = sid

        # 追加運転日で全部足す
        for addday in ui.drivedays:
            addday_date = datetime.strptime(str(addday), '%Y%m%d')
            data2.write('{0},{1},1\n'.format(TextHosei(sid), '{0}{1:02}{2:02}'.format(int(addday_date.year), int(addday_date.month), int(addday_date.day))))

        # 定義重複防止(ver1のみ。ver2では支援ツール側で同名定義不可能としている)
        if Mode == 'ver1':
            if sid in chkDup:
                print('ブック名:{0} シート「④運航日」 定義"{1}"が重複しています。先に定義されている運航日が使われます'.format(ui.kouro, ui.name))
                continue

        if Mode == 'ver2':
            if sid in chkDup:
                continue

        # ver2対応では全ての定義は運転日になる。曜日運転がすべて0であればcalendar.txtの定義は不要
        if ','.join(ui.week[:7]) != '0,0,0,0,0,0,0' or Mode == 'ver1':
            data1.write('{0},{1},{2},{3}\n'.format(TextHosei(sid), ','.join(ui.week[:7]), ui.startdate, ui.enddate))

        nRestdays.sort()
        for rest in nRestdays:
            rest_date = datetime.strptime(str(rest), '%Y%m%d')
            data2.write('{0},{1},2\n'.format(TextHosei(sid), '{0}{1:02}{2:02}'.format(int(rest_date.year), int(rest_date.month), int(rest_date.day))))

        cnt += 1
        chkDup.append(sid)
    data1.close()
    data2.close()

    data = open('{0}/payload.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'paylaod_id,car_allowed,car_payload_limit,scooter_allowed,motorcycle_allowed,large_motorcycle_allowed,payload_desc\n')

    # payload.txt
    kouro2pl = dict()
    kouro2bicycle = dict()
    usedpayload = list()

    for i in range(len(TokusyuList)):
        # 重複を省くかどうか？省くならそれ用の判定処理が必要
        # plid = 'PL{0:03}'.format(i+1)
        plid = '{0}積載'.format(kouro2id[TokusyuList[i].kouro])

        # 仮にある船体が2つの航路で用いられている場合、後の方は無視する
        if plid in usedpayload:
            continue

        usedpayload.append(plid)

        df = '0'
        flgcar = df
        flgbike = df
        flgsco = df
        flgmor_s = df
        flgmor_b = df

        maxsize = -1
        CarRide = False
        Unknown = False
        for j in range(11):
            if TokusyuList[i].larges[j] > 0:
                maxsize = j
                CarRide = True
            if TokusyuList[i].larges[j] == 0:  # 不明状態
                Unknown = True

        # 持ち込み可
        if CarRide == True:
            flgcar = '1'
            maxsize += 3
            maxsize = float(maxsize)
        # 持ち込み不可だが不明もある時は不明
        elif CarRide == False and Unknown == True:
            flgcar = '0'  # デフォルト
            maxsize = ''
        # 明確に不可
        else:
            flgcar = '2'
            maxsize = ''

        for j in range(12, 16):
            # 持ち込み可能
            if TokusyuList[i].larges[j] > 0:
                if j == 12:
                    flgbike = '1'
                if j == 13:
                    flgsco = '1'
                if j == 14:
                    flgmor_s = '1'
                if j == 15:
                    flgmor_b = '1'

            # 持ち込み不可能
            if TokusyuList[i].larges[j] < 0:
                if j == 12:
                    flgbike = '2'
                if j == 13:
                    flgsco = '2'
                if j == 14:
                    flgmor_s = '2'
                if j == 15:
                    flgmor_b = '2'

        comment = '{0}／{1}'.format(TokusyuList[i].comment1, TokusyuList[i].comment2)
        # comment = ''

        data.write('{0},{1},{2},{3},{4},{5},{6}\n'.format(TextHosei(plid), flgcar, maxsize, flgsco, flgmor_s, flgmor_b,
                                                          comment))

        kouro2pl[TokusyuList[i].kouro] = plid
        kouro2bicycle[TokusyuList[i].kouro] = flgbike

    data.close()

    # trips.txt & stop_times.txt
    data1 = open('{0}/trips.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data2 = open('{0}/stop_times.txt'.format(output), 'wt', encoding='utf-8', newline='\n')

    data1.write(
        'route_id,service_id,trip_id,trip_headsign,trip_short_name,direction_id,block_id,shape_id,wheelchair_accessible,bikes_allowed,payload_id,ships_id\n')
    data2.write(
        'trip_id,arrival_time,departure_time,stop_id,stop_sequence,stop_headsign,pickup_type,drop_off_type,shape_dist_traveled,timepoint\n')

    usedshapes = list()
    usedships = list()

    tidcnt = 1
    for i in range(len(DaiyaList)):
        for j in range(len(DaiyaList[i])):
            Daiya = DaiyaList[i][j]
            direction = 0

            # 空行対策
            if len(Daiya.ports) == 0:
                continue

            # クラスのリストを着時刻でソートする
            sortTimes = sorted(Daiya.ports)

            # 並び変わったら上りとする
            if sortTimes[0].arv != Daiya.ports[0].arv:
                direction = 1

            route_id = kouro2id[Daiya.kouro]
            service_id = ''
            if Mode == 'ver1':
                service_id = unten2id[Daiya.day]
            if Mode == 'ver2':
                service_id = Daiya.day

            headsign = MakeTripHeadsign(sortTimes, port2id, id2port)
            short_name = Daiya.ship
            trip_id = '[{0:03}]{1}{2}'.format(tidcnt, short_name, service_id)
            block_id = ''

            # shapes追加。但し、ファイル存在チェックをしてから行う
            if ver2dir != '' and Daiya.shape != '':
                shape_id = Daiya.shape
                if os.path.exists('{0}/shapes/{1}.geojson'.format(ver2dir, shape_id)):
                    if shape_id not in usedshapes:
                        usedshapes.append(shape_id)
                else:
                    print('航海ルートの指定ファイル"{0}.geojson"が設定されていますが、指定のフォルダに存在しません'.format(shape_id))
                    AlertFlg = 1
            else:
                shape_id = ''
            wheel = '0'
            bike = kouro2bicycle[Daiya.kouro]
            payload = kouro2pl[Daiya.kouro]

            # ver2
            ships_id = Daiya.ship.split('：')[0]

            if ships_id not in usedships:
                usedships.append(ships_id)

            data1.write('{0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11}\n'.format(
                TextHosei(route_id), TextHosei(service_id), TextHosei(trip_id), headsign,
                TextHosei(short_name), direction, block_id, shape_id, wheel, bike, TextHosei(payload), ships_id))

            # 着時刻でソートしているが、その着時刻のペアの発時刻が不正な場合次の時刻に繋がらない。
            # 入力データミスの可能性が高い。コンバータ側での補正をするのは好ましくないため、警告する
            for j in range(len(sortTimes) - 1):
                if sortTimes[j].dep > sortTimes[j + 1].arv:
                    print('trip_id:{0} {1}～{2}間 時刻表に不正があります。確認してください'.format(
                        trip_id, sortTimes[j].name.split('※')[0], sortTimes[j + 1].name.split('※')[0]))
                    AlertFlg = 1

            tidcnt += 1

            # 時刻表作成
            for j in range(len(sortTimes)):
                pick = 0
                drop = 0

                # 複雑な乗降禁止がありえるがGTFSの仕様上作れない
                if j == 0:
                    drop = 1
                if j == len(sortTimes) - 1:
                    pick = 1

                data2.write('{0},{1},{2},{3},{4},{5},{6},{7},{8},{9}\n'.format(
                    TextHosei(trip_id), gtfs_hhmmss(sortTimes[j].arv), gtfs_hhmmss(sortTimes[j].dep),
                    TextHosei(port2id[sortTimes[j].name.split('※')[0]]), j, '', pick, drop, '', ''))
    data1.close()
    data2.close()

    # ships.txt
    data = open('{0}/ships.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'ships_id,gross_tonnage,engine_power,number_of_engine,speed,passenger_capacity,shipping_truck,shipping_car,launch_date,equipment\n')
    for ship in usedships:
        si = next((x for x in ShipList if x.name == ship), None)

        if si == None:
            continue

        data.write('{0},{1},{2},{3},{4},{5},{6},{7},{8},{9}\n'.format(
            si.name, si.weight, si.power, si.engine_cnt, si.speed, si.person_cap, si.truck_cap, si.car_cap, si.launch,
            si.equipment))

    data.close()

    # fare_rules.txt & fare_attributes.txt
    data1 = open('{0}/fare_rules.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data2 = open('{0}/fare_attributes.txt'.format(output), 'wt', encoding='utf-8', newline='\n')

    data1.write('fare_id,route_id,origin_id,destination_id,contains_id\n')
    data2.write('fare_id,price,currency_type,payment_method,transfers,agency_id,transfer_duration,cabin_name\n')
    for i in range(len(UntinList)):
        UsedFareID = list()
        for j in range(len(UntinList[i])):
            fare = UntinList[i][j]

            # fare_id = 'F{0:06}'.format(fare.untin)
            fare_id = '[{0:06}]{1}_{2}～{3}'.format(fare.untin, TextHosei(kouro2id[fare.kouro]), port2id[fare.port1], port2id[fare.port2])

            if fare_id not in UsedFareID:
                UsedFareID.append(fare_id)
                data2.write(
                    '{0},{1},{2},{3},{4},{5},{6},{7}\n'.format(TextHosei(fare_id), fare.untin, 'JPY', 1, 0, ci.aid, '',
                                                               fare.cabin_name))

            # 明記されていないが三角表ベースから作ると往復分になる
            data1.write('{0},{1},{2},{3},{4}\n'.format(TextHosei(fare_id), TextHosei(kouro2id[fare.kouro])
                                                       , TextHosei(port2id[fare.port1]), TextHosei(port2id[fare.port2]),
                                                       ''))
            data1.write('{0},{1},{3},{2},{4}\n'.format(TextHosei(fare_id), TextHosei(kouro2id[fare.kouro])
                                                       , TextHosei(port2id[fare.port1]), TextHosei(port2id[fare.port2]),
                                                       ''))
    data1.close()
    data2.close()

    # feed_info.txt
    feed_ver2 = '2.0'
    data = open('{0}/feed_info.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'feed_publisher_name,feed_publisher_url,feed_lang,feed_start_date,feed_end_date,feed_version,feed_contact_email,feed_contact_url\n')

    data.write('{0},{1},{2},{3},{4},{5},{6},{7}\n'.format(
        TextHosei(ci.name), TextHosei(ci.topurl), 'ja', MinDate, MaxDate, feed_ver2, TextHosei(ci.email), ''))
    data.close()

    # alertデータの出力
    # ヘッダー
    Message = gtfs_realtime_pb2.FeedMessage()
    Message.header.gtfs_realtime_version = "2.0"
    Message.header.incrementality = gtfs_realtime_pb2.FeedHeader.FULL_DATASET

    dt = datetime.now()
    Message.header.timestamp = int(time.mktime(dt.timetuple()))

    cnt = 0
    for al in AlertList:
        cnt += 1
        Entity = gtfs_realtime_pb2.FeedEntity()
        Entity.id = '{0:03d}_{1}'.format(cnt, al.route)

        Active_Period = Entity.alert.active_period.add()
        Active_Period.start = al.start
        Active_Period.end = al.end

        Informed_Entity = Entity.alert.informed_entity.add()
        Informed_Entity.route_id = al.route

        Entity.alert.cause = al.cause
        Entity.alert.effect = al.effect

        translation = Entity.alert.header_text.translation.add()
        translation.text = al.header
        translation.language = 'ja'

        translation = Entity.alert.description_text.translation.add()
        translation.text = al.description
        translation.language = 'ja'

        Message.entity.extend([Entity])

    aloutput = open('gtfs_alert.pb', 'wb')
    aloutput.write(Message.SerializeToString())
    aloutput.close()

    # jsonにdumpしたファイルも出力する
    json_str = MessageToJson(Message)
    json_dic = json.loads(json_str)

    save = open('gtfs_alert.json', 'w')
    json.dump(json_dic, save, ensure_ascii=False, indent=4, sort_keys=True)
    save.close()

    # shapesファイル出力
    if len(usedshapes) > 0:
        data = open('{0}/shapes.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
        data.write('shape_id,shape_pt_lat,shape_pt_lon,shape_pt_sequence,shape_dist_traveled\n')

        for shape in usedshapes:
            # ファイルがあるのは確認済み
            geo_path = '{0}/shapes/{1}.geojson'.format(ver2dir, shape)
            geojson = open(geo_path, 'r', encoding='utf-8')
            jsondict = json.load(geojson)

            for sp in jsondict['features']:
                geo_cnt = 0
                for geo in sp['geometry']['coordinates']:
                    data.write('{0},{1},{2},{3},\n'.format(shape, geo[1], geo[0], geo_cnt))
                    geo_cnt += 1
        data.close()

    # payload_fare_attributes
    data = open('{0}/payload_fare_attributes.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write(
        'payload_fare_id,price,currency_type,payment_method,transfers,agency_id,transfer_duration,payload_name,including_price_fare_id\n')

    for i in range(len(TokusyuList)):
        for j in range(len(TokusyuList[i].ids)):
            # 料金が0の時はcontinue
            if TokusyuList[i].larges[j] <= 0:
                continue

            # ドライバー料金を含まない場合はincluding_price_fare_idは空白の1つで終了
            if TokusyuList[i].includings[j] == 1:
                data.write('{0},{1},{2},{3},{4},{5},{6},{7},{8}\n'.format(
                    TokusyuList[i].ids[j],
                    TokusyuList[i].larges[j],
                    'JPY',
                    '1',
                    '0',
                    ci.aid,
                    '',
                    TokusyuList[i].ids[j].split('|')[1],
                    ''
                ))

                continue

            # including_fare_idの仕様上、すべてのfare_id分作る必要がある。
            for k in range(len(UsedFareID)):
                data.write('{0},{1},{2},{3},{4},{5},{6},{7},{8}\n'.format(
                    TokusyuList[i].ids[j],
                    TokusyuList[i].larges[j],
                    'JPY',
                    '1',
                    '0',
                    ci.aid,
                    '',
                    TokusyuList[i].ids[j].split('|')[1],
                    UsedFareID[k]
                ))

    data.close()

    # payload_fare_rules
    data = open('{0}/payload_fare_rules.txt'.format(output), 'wt', encoding='utf-8', newline='\n')
    data.write('payload_fare_id,route_id,origin_id,destination_id,contains_id\n')

    UsedPare = set()

    for i in range(len(UntinList)):
        for j in range(len(UntinList[i])):
            fare = UntinList[i][j]

            up = '{0}|{1}|{2}'.format(fare.kouro, fare.port1, fare.port2)

            # 組み合わせとして登録済みであれば重複しているので不要
            if up not in UsedPare:
                UsedPare.add(up)
            else:
                continue

            if fare.kouro == TokusyuList[i].kouro:
                for k in range(len(TokusyuList[i].ids)):
                    if TokusyuList[i].larges[k] <= 0:
                        continue

                    data.write('{0},{1},{2},{3},{4}\n'.format(
                        TokusyuList[i].ids[k],
                        TextHosei(kouro2id[fare.kouro]),
                        TextHosei(port2id[fare.port1]),
                        TextHosei(port2id[fare.port2]),
                        ''
                    ))

                    data.write('{0},{1},{2},{3},{4}\n'.format(
                        TokusyuList[i].ids[k],
                        TextHosei(kouro2id[fare.kouro]),
                        TextHosei(port2id[fare.port2]),
                        TextHosei(port2id[fare.port1]),
                        ''
                    ))

    data.close()

    # ※つき停留所を削除する
    ComePortDelete(output)

    # 循環系統の行先補正
    mk_headsign(output)

    # zipに圧縮
    if os.path.exists(output):
        zipname = '{0}'.format(output)
        shutil.make_archive(zipname, 'zip', root_dir=output)

    if AlertFlg == 1:
        print(' >標準フォーマットデータの作成が完了しましたが、問題があります\n >本ファイルに記載されているエラーを確認してください')
        print(' >確認したらこのログファイルを開いているプログラム（メモ帳など）は閉じて問題ありません')
        print(' >エラーを見直す場合は、log.txtをダブルクリックしてください')
    else:
        print(' >標準フォーマットデータの作成が成功しました')
